/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

/*
 * DBFastClockWebServer.java
 *
 * Created on 2013-mar-10, 05:46:00
 */
package dbfastclockwebserver;

import java.awt.Component;
import java.awt.Container;
import java.awt.event.ItemEvent;
import java.awt.event.WindowEvent;
import java.awt.event.WindowListener;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.FileWriter;
import java.io.FilenameFilter;
import java.io.IOException;
import java.lang.reflect.InvocationTargetException;
import java.net.InetAddress;
import java.net.MalformedURLException;
import java.net.NetworkInterface;
import java.net.SocketException;
import java.net.URL;
import java.net.URLClassLoader;
import java.net.UnknownHostException;
import java.text.MessageFormat;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.List;
import java.util.Locale;
import java.util.Properties;
import java.util.ResourceBundle;
import java.util.concurrent.ScheduledThreadPoolExecutor;
import java.util.concurrent.TimeUnit;
import java.util.regex.Matcher;
import java.util.regex.Pattern;
import javax.swing.SwingUtilities;
import javax.swing.border.Border;

/**
 *
 * @author Daniels laptop
 */
public class DBFastClockWebServer extends javax.swing.JFrame implements Runnable, WindowListener
{
	static final int COLOR_GREEN_FOREGROUND_BLACK_BACKGROUND = 0;
	static final int COLOR_WHITE_FOREGROUND_BLACK_BACKGROUND = 1;
	static final int COLOR_BLACK_FOREGROUND_WHITE_BACKGROUND = 2;
	
	
	List<Clock> clockList = new ArrayList<Clock>();
	MySignal mySignal = new MySignal(clockList);
	Properties prop = new Properties();
	TCPClientComm tcpClientComm = new TCPClientComm(mySignal);
	HTTPServerComm httpServerComm;
	boolean programIsInitialized = false;
	private FileWriter debugFile;
	
	ArrayList<String> hostIPAddresses = new ArrayList<String>();
	int colorSetting = COLOR_GREEN_FOREGROUND_BLACK_BACKGROUND;
	
	List<ResourceBundle> localizationResourceBundleList = new ArrayList<ResourceBundle>();
	ResourceBundle currentLocalizationResourceBundle = null;
	String currentLanguage = null;

	
	/** Creates new form DBFastClockWebServer */
	public DBFastClockWebServer()
	{
		InetAddress localHost = null;
		
		initComponents();
		
		mySignal.dbFastClockWebServer = this;
		
		readProperties();
		
		Clock menu = ClockMenu.GetClock(mySignal);
		
		mySignal.setMenuClock(menu);
		
		clockList.add( menu );
		clockList.add( HTMLClock.GetClock(mySignal) );
		clockList.add( HTMLTextClock.GetClock(mySignal) );
		clockList.add( HTML_SVG_Clock.GetClock(mySignal) );
		
		String clockProperty = prop.getProperty("clock");
		
		jComboBox_ClockType.removeAllItems();
		
		for (Clock clock : clockList)
		{
			String clock_name_template;
			
			try
			{
				if (currentLanguage != null)
				{
					clock_name_template = currentLocalizationResourceBundle.getString("clocktype_" + clock.getName());
				}
				else
				{
					clock_name_template = "clocktype_" + clock.getName();
				}
			}
			catch (java.util.MissingResourceException ex)
			{
				clock_name_template = clock.getName() + ": {0}";
				
				String error_key = ex.getKey();
				String error_msg_template = "The language file for \"{0}\" is missing the key \"{1}\"\n";
				
				Object[] messageArguments =
				{
					currentLanguage,
					ex.getKey(),
					error_key
				};
				
				MessageFormat msgFormatter = new MessageFormat("");
				msgFormatter.setLocale(currentLocalizationResourceBundle.getLocale());
				msgFormatter.applyPattern(error_msg_template);
				jTextArea_Output.append(msgFormatter.format(messageArguments));
			}
			
			Object[] messageArguments =
			{
				clock.getName()
			};
			
			MessageFormat msgFormatter = new MessageFormat("");
			if (currentLanguage != null)
			{
				msgFormatter.setLocale(currentLocalizationResourceBundle.getLocale());
			}
			msgFormatter.applyPattern(clock_name_template);
			
			jComboBox_ClockType.addItem(msgFormatter.format(messageArguments));
			
			if (clockProperty != null)
			{
				if (clockProperty.matches(clock.getName()))
				{
					jComboBox_ClockType.setSelectedIndex(jComboBox_ClockType.getItemCount() - 1);
					mySignal.setClock(clock);
				}
			}
		}
		
		
		try
		{
			localHost = InetAddress.getLocalHost();
		}
		catch (UnknownHostException ex)
		{
			// Do nothing
		}
		
		hostIPAddresses.clear();
		jComboBox_IPaddress.removeAllItems();
		
		try
		{
			Enumeration e = NetworkInterface.getNetworkInterfaces();
			
			while (e.hasMoreElements())
			{
				NetworkInterface n = (NetworkInterface) e.nextElement();
				Enumeration ee = n.getInetAddresses();
				while (ee.hasMoreElements())
				{
					InetAddress i = (InetAddress) ee.nextElement();
					
//					System.out.println(i.getHostAddress());
					
					if (n.isUp())
					{
						hostIPAddresses.add(i.getHostAddress());
						jComboBox_IPaddress.addItem(n.getDisplayName()+":   "+i.getHostAddress());
						
						if ( (localHost != null) && (localHost.equals(i)) )
							jComboBox_IPaddress.setSelectedIndex(jComboBox_IPaddress.getItemCount()-1);
					}
				}
			}
		}
		catch (SocketException ex)
		{
			System.out.println(ex.getMessage());
		}
		
		
		if (mySignal.getClock() == null)
		{
			mySignal.setClock(clockList.get(0));
		}
		
		
		mySignal.setClockTime(MySignal.CLOCK_TYPE__FAST_CLOCK, true, MySignal.WEEKDAY_NONE, 0, 0, 0, 1.0);
//		mySignal.setIPaddressAndPort("192.168.1.101", 2500, 80);
        jTextField_ClientIPaddress.setText(prop.getProperty("client_ip_address"));
        jTextField_ClientTCPport.setText(prop.getProperty("client_tcp_port"));
		mySignal.setIPaddressAndPort(prop.getProperty("client_ip_address"), Integer.parseInt(prop.getProperty("client_tcp_port","-1")), Integer.parseInt(prop.getProperty("server_http_port","-1")));
//		mySignal.setIPaddressAndPort("192.168.1.101", 2500, 2502);
		
		try
		{
			// Create file 
			debugFile = new FileWriter("debug.txt");
			mySignal.debug = new BufferedWriter(debugFile);
		}
		catch (IOException ex)
		{
//			Logger.getLogger(DBFastClock.class.getName()).log(Level.SEVERE, null, ex);
		}
		
		addWindowListener(this);
		
		pack();		// Update dialog
	}

	/** This method is called from within the constructor to
	 * initialize the form.
	 * WARNING: Do NOT modify this code. The content of this method is
	 * always regenerated by the Form Editor.
	 */
	@SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        jPanel9 = new javax.swing.JPanel();
        jLabel12 = new javax.swing.JLabel();
        jTextField_ClientIPaddress = new javax.swing.JTextField();
        jLabel13 = new javax.swing.JLabel();
        jTextField_ClientTCPport = new javax.swing.JTextField();
        jLabel14 = new javax.swing.JLabel();
        jTextField_ServerHTTPport = new javax.swing.JTextField();
        jLabel2 = new javax.swing.JLabel();
        jComboBox_IPaddress = new javax.swing.JComboBox();
        jPanel7 = new javax.swing.JPanel();
        jComboBox_Language = new javax.swing.JComboBox();
        jLabel16 = new javax.swing.JLabel();
        jComboBox_ClockType = new javax.swing.JComboBox();
        jLabel1 = new javax.swing.JLabel();
        jButton_SaveSettings = new javax.swing.JButton();
        jPanel1 = new javax.swing.JPanel();
        jScrollPane1 = new javax.swing.JScrollPane();
        jTextArea_Output = new javax.swing.JTextArea();

        setDefaultCloseOperation(javax.swing.WindowConstants.EXIT_ON_CLOSE);

        jPanel9.setBorder(javax.swing.BorderFactory.createTitledBorder("IP-adress och port"));
        jPanel9.setName("group_ip_address_and_port"); // NOI18N

        jLabel12.setText("IP-adress:");
        jLabel12.setName("ip_address"); // NOI18N

        jTextField_ClientIPaddress.setText("127.0.0.1");

        jLabel13.setText("TCP-port:");
        jLabel13.setName("tcp_port"); // NOI18N

        jTextField_ClientTCPport.setText("2500");

        jLabel14.setText("HTTP-port:");
        jLabel14.setName("tcp_port"); // NOI18N

        jTextField_ServerHTTPport.setText("80");

        jLabel2.setText("IP-adress");
        jLabel2.setName("ip_address"); // NOI18N

        jComboBox_IPaddress.setModel(new javax.swing.DefaultComboBoxModel(new String[] { "Item 1", "Item 2", "Item 3", "Item 4" }));

        javax.swing.GroupLayout jPanel9Layout = new javax.swing.GroupLayout(jPanel9);
        jPanel9.setLayout(jPanel9Layout);
        jPanel9Layout.setHorizontalGroup(
            jPanel9Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel9Layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(jPanel9Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(jLabel12, javax.swing.GroupLayout.Alignment.TRAILING)
                    .addComponent(jLabel13, javax.swing.GroupLayout.Alignment.TRAILING)
                    .addComponent(jLabel2, javax.swing.GroupLayout.Alignment.TRAILING)
                    .addComponent(jLabel14, javax.swing.GroupLayout.Alignment.TRAILING))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                .addGroup(jPanel9Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addGroup(jPanel9Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING, false)
                        .addComponent(jTextField_ClientTCPport)
                        .addComponent(jTextField_ClientIPaddress, javax.swing.GroupLayout.PREFERRED_SIZE, 107, javax.swing.GroupLayout.PREFERRED_SIZE))
                    .addComponent(jComboBox_IPaddress, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(jTextField_ServerHTTPport, javax.swing.GroupLayout.PREFERRED_SIZE, 82, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );
        jPanel9Layout.setVerticalGroup(
            jPanel9Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel9Layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(jPanel9Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(jLabel12)
                    .addComponent(jTextField_ClientIPaddress, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(jPanel9Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(jTextField_ClientTCPport, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(jLabel13))
                .addGap(18, 18, 18)
                .addGroup(jPanel9Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(jLabel2)
                    .addComponent(jComboBox_IPaddress, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(jPanel9Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(jLabel14)
                    .addComponent(jTextField_ServerHTTPport, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addContainerGap(17, Short.MAX_VALUE))
        );

        jPanel7.setBorder(javax.swing.BorderFactory.createTitledBorder("Allmänt"));
        jPanel7.setName("group_common"); // NOI18N

        jComboBox_Language.setModel(new javax.swing.DefaultComboBoxModel(new String[] { "aaa", "bbb", "ccc" }));
        jComboBox_Language.addItemListener(new java.awt.event.ItemListener() {
            public void itemStateChanged(java.awt.event.ItemEvent evt) {
                jComboBox_LanguageItemStateChanged(evt);
            }
        });

        jLabel16.setText("Språk");
        jLabel16.setName("language"); // NOI18N

        jComboBox_ClockType.setModel(new javax.swing.DefaultComboBoxModel(new String[] { "Analog", "Digital" }));
        jComboBox_ClockType.addItemListener(new java.awt.event.ItemListener() {
            public void itemStateChanged(java.awt.event.ItemEvent evt) {
                jComboBox_ClockTypeItemStateChanged(evt);
            }
        });

        jLabel1.setText("Klocka:");
        jLabel1.setName("clock_style"); // NOI18N

        javax.swing.GroupLayout jPanel7Layout = new javax.swing.GroupLayout(jPanel7);
        jPanel7.setLayout(jPanel7Layout);
        jPanel7Layout.setHorizontalGroup(
            jPanel7Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel7Layout.createSequentialGroup()
                .addGap(42, 42, 42)
                .addGroup(jPanel7Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING)
                    .addComponent(jLabel1)
                    .addComponent(jLabel16))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                .addGroup(jPanel7Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(jComboBox_Language, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(jComboBox_ClockType, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addContainerGap())
        );
        jPanel7Layout.setVerticalGroup(
            jPanel7Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel7Layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(jPanel7Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(jLabel16)
                    .addComponent(jComboBox_Language, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                .addGroup(jPanel7Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(jLabel1)
                    .addComponent(jComboBox_ClockType, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addContainerGap(76, Short.MAX_VALUE))
        );

        jButton_SaveSettings.setText("Save");
        jButton_SaveSettings.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButton_SaveSettingsActionPerformed(evt);
            }
        });

        jPanel1.setBorder(javax.swing.BorderFactory.createTitledBorder("Messages"));

        jTextArea_Output.setColumns(20);
        jTextArea_Output.setRows(5);
        jScrollPane1.setViewportView(jTextArea_Output);

        javax.swing.GroupLayout jPanel1Layout = new javax.swing.GroupLayout(jPanel1);
        jPanel1.setLayout(jPanel1Layout);
        jPanel1Layout.setHorizontalGroup(
            jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel1Layout.createSequentialGroup()
                .addContainerGap()
                .addComponent(jScrollPane1, javax.swing.GroupLayout.DEFAULT_SIZE, 327, Short.MAX_VALUE)
                .addContainerGap())
        );
        jPanel1Layout.setVerticalGroup(
            jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel1Layout.createSequentialGroup()
                .addComponent(jScrollPane1, javax.swing.GroupLayout.DEFAULT_SIZE, 127, Short.MAX_VALUE)
                .addContainerGap())
        );

        javax.swing.GroupLayout layout = new javax.swing.GroupLayout(getContentPane());
        getContentPane().setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addContainerGap()
                .addComponent(jPanel9, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addGap(18, 18, 18)
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(jButton_SaveSettings)
                    .addGroup(layout.createSequentialGroup()
                        .addComponent(jPanel7, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addGap(18, 18, 18)
                        .addComponent(jPanel1, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)))
                .addContainerGap())
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING)
                    .addComponent(jPanel1, javax.swing.GroupLayout.Alignment.LEADING, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                    .addGroup(javax.swing.GroupLayout.Alignment.LEADING, layout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING, false)
                        .addComponent(jPanel9, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                        .addComponent(jPanel7, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)))
                .addGap(18, 18, 18)
                .addComponent(jButton_SaveSettings)
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );

        pack();
    }// </editor-fold>//GEN-END:initComponents

	private void jComboBox_LanguageItemStateChanged(java.awt.event.ItemEvent evt)//GEN-FIRST:event_jComboBox_LanguageItemStateChanged
	{//GEN-HEADEREND:event_jComboBox_LanguageItemStateChanged
		try
		{
			// We get this event both for select and deselect item
			if (evt.getStateChange() == ItemEvent.SELECTED)
			{ 				// Make sure the languages has been loaded
				if (!localizationResourceBundleList.isEmpty())
				{
					currentLocalizationResourceBundle = localizationResourceBundleList.get(jComboBox_Language.getSelectedIndex());
				}
				currentLanguage = evt.getItem().toString();
				localizeLabels(this);
				int selectedClockType = jComboBox_ClockType.getSelectedIndex();
				jComboBox_ClockType.removeAllItems();
				for (Clock clock : mySignal.getClockList())
					jComboBox_ClockType.addItem(getLocalizedName("clocktype_", clock.getName()));
				jComboBox_ClockType.setSelectedIndex(selectedClockType);
				pack();		// Se till att dialogen är tillräckligt stor efter "localizeLabels"
//					updateStartPauseRUTClockButton();
			}
			prop.setProperty("language", evt.getItem().toString());
		}
		catch (Throwable ex)
		{
//			addInfoMessageLn(StringUtils.getStackTrace(ex));
		}
	}//GEN-LAST:event_jComboBox_LanguageItemStateChanged

	private void jButton_SaveSettingsActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_jButton_SaveSettingsActionPerformed
	{//GEN-HEADEREND:event_jButton_SaveSettingsActionPerformed
        prop.setProperty("client_ip_address", jTextField_ClientIPaddress.getText());
        prop.setProperty("client_tcp_port",jTextField_ClientTCPport.getText());
        prop.setProperty("server_http_port",jTextField_ServerHTTPport.getText());
		
		writeProperties();
	}//GEN-LAST:event_jButton_SaveSettingsActionPerformed

	private void jComboBox_ClockTypeItemStateChanged(java.awt.event.ItemEvent evt)//GEN-FIRST:event_jComboBox_ClockTypeItemStateChanged
	{//GEN-HEADEREND:event_jComboBox_ClockTypeItemStateChanged
		// We get this event both for select and deselect item
		if (evt.getStateChange() == ItemEvent.SELECTED)
		{
			// Make sure the languages has been loaded
			if (! clockList.isEmpty())
			{
				mySignal.setClock( clockList.get(jComboBox_ClockType.getSelectedIndex()) );
				
				prop.setProperty("clock", mySignal.getClock().getName());
			}
		}
	}//GEN-LAST:event_jComboBox_ClockTypeItemStateChanged

	/**
	 * @param args the command line arguments
	 */
	public static void main(String args[])
	{
		/* Set the Nimbus look and feel */
		//<editor-fold defaultstate="collapsed" desc=" Look and feel setting code (optional) ">
        /* If Nimbus (introduced in Java SE 6) is not available, stay with the default look and feel.
		 * For details see http://download.oracle.com/javase/tutorial/uiswing/lookandfeel/plaf.html 
		 */
		try
		{
			for (javax.swing.UIManager.LookAndFeelInfo info : javax.swing.UIManager.getInstalledLookAndFeels())
			{
				if ("Nimbus".equals(info.getName()))
				{
					javax.swing.UIManager.setLookAndFeel(info.getClassName());
					break;
				}
			}
		}
		catch (ClassNotFoundException ex)
		{
			java.util.logging.Logger.getLogger(DBFastClockWebServer.class.getName()).log(java.util.logging.Level.SEVERE, null, ex);
		}
		catch (InstantiationException ex)
		{
			java.util.logging.Logger.getLogger(DBFastClockWebServer.class.getName()).log(java.util.logging.Level.SEVERE, null, ex);
		}
		catch (IllegalAccessException ex)
		{
			java.util.logging.Logger.getLogger(DBFastClockWebServer.class.getName()).log(java.util.logging.Level.SEVERE, null, ex);
		}
		catch (javax.swing.UnsupportedLookAndFeelException ex)
		{
			java.util.logging.Logger.getLogger(DBFastClockWebServer.class.getName()).log(java.util.logging.Level.SEVERE, null, ex);
		}
		//</editor-fold>

		/* Create and display the form */
		java.awt.EventQueue.invokeLater(new Runnable()
		{

			public void run()
			{
				new DBFastClockWebServer().setVisible(true);
			}
		});
	}
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JButton jButton_SaveSettings;
    private javax.swing.JComboBox jComboBox_ClockType;
    private javax.swing.JComboBox jComboBox_IPaddress;
    private javax.swing.JComboBox jComboBox_Language;
    private javax.swing.JLabel jLabel1;
    private javax.swing.JLabel jLabel12;
    private javax.swing.JLabel jLabel13;
    private javax.swing.JLabel jLabel14;
    private javax.swing.JLabel jLabel16;
    private javax.swing.JLabel jLabel2;
    private javax.swing.JPanel jPanel1;
    private javax.swing.JPanel jPanel7;
    private javax.swing.JPanel jPanel9;
    private javax.swing.JScrollPane jScrollPane1;
    private javax.swing.JTextArea jTextArea_Output;
    private javax.swing.JTextField jTextField_ClientIPaddress;
    private javax.swing.JTextField jTextField_ClientTCPport;
    private javax.swing.JTextField jTextField_ServerHTTPport;
    // End of variables declaration//GEN-END:variables



	void localizeLabels(Container c)
	{
		Component[] components = c.getComponents();
		for (Component com : components)
		{
			if (com instanceof javax.swing.JLabel)
			{
				((javax.swing.JLabel) com).setText(getLocalizedName("label_", com.getName()));
			}
			else if (com instanceof javax.swing.JButton)
			{
				String name = com.getName();
				if (name != null)
					((javax.swing.JButton) com).setText(getLocalizedName("label_", name));
//				else
//					jTextArea_Output.append("Name is null\n");
			}
			else if (com instanceof javax.swing.JComboBox)
			{
				// Do nothing
			}
			else if (com instanceof javax.swing.JScrollBar)
			{
				// Do nothing
			}
			else if (com instanceof javax.swing.JCheckBox)
			{
				((javax.swing.JCheckBox) com).setText(getLocalizedName("label_", com.getName()));
			}
			else if (com instanceof javax.swing.JPanel)
			{
				Border border = ((javax.swing.JPanel) com).getBorder();
				
				if (border instanceof javax.swing.border.TitledBorder)
				{
					String name = com.getName();
					if (name != null)
						((javax.swing.border.TitledBorder) border).setTitle(getLocalizedName("label_", name));
				}
				
				if (com instanceof Container)
				{
					localizeLabels((Container) com);
				}
			}
			else
			{
				if (com instanceof Container)
				{
					localizeLabels((Container) com);
				}
			}
        }
	}
	
	
	public String getLocalizedName(String key_prefix, String key)
	{
		return getLocalizedName(key_prefix, key, null);
	}
	
	
	public String getLocalizedName(String key_prefix, String key, String parameter)
	{
		String value = key;
		
		if (currentLanguage == null)
		{
			if (parameter != null)
				return key + ": " + parameter;
			else
				return key;
		}
		
		if (key == null)
		{
			try
			{
				mySignal.debug.write("Software error in function getLocalizedName: The key is null. key_prefix: "+key_prefix+"\n");
//				jTextArea_Output.append(formatter.format(messageArguments));
			}
			catch (IOException ex1)
			{
//					Logger.getLogger(DBFastClock.class.getName()).log(Level.SEVERE, null, ex1);
			}
//			addInfoMessageLn("Software error in function getLocalizedName: The key is null. key_prefix: "+key_prefix);
			value = "<null>";
		}
		else if (! key.matches("no_translate"))		// Some labels should not be translated here, for example "am / pm"-label.
		{
			try
			{
				value = currentLocalizationResourceBundle.getString(key_prefix + key);
				
				if (parameter != null)
				{
//					String key_template = currentLocalizationResourceBundle.getString(key_prefix + key)+"\n";
					
					Object[] messageArguments =
					{
						parameter
					};

					MessageFormat formatter = new MessageFormat("");
					formatter.setLocale(currentLocalizationResourceBundle.getLocale());
					formatter.applyPattern(value);
					value = formatter.format(messageArguments);
				}
			}
			catch (java.util.MissingResourceException ex)
			{
				String error_msg_template;
				String error_key = "";
				
				try
				{
					error_msg_template = currentLocalizationResourceBundle.getString("error_missing_error_msg_missing_localization_key")+"\n";
				}
				catch (java.util.MissingResourceException ex2)
				{
					error_key = ex2.getKey();
					error_msg_template	=	"The language file for \"{0}\" is missing the key \"{2}\"\n" +
											"The language file for \"{0}\" is missing the key \"{1}\"\n";
				}
				
				Object[] messageArguments =
				{
					currentLanguage,
					ex.getKey(),
					error_key
				};
				
				MessageFormat formatter = new MessageFormat("");
				formatter.setLocale(currentLocalizationResourceBundle.getLocale());
				formatter.applyPattern(error_msg_template);
				try
				{
					mySignal.debug.write(formatter.format(messageArguments)+"\n");
//					jTextArea_Output.append(formatter.format(messageArguments));
				}
				catch (IOException ex1)
				{
//					Logger.getLogger(DBFastClock.class.getName()).log(Level.SEVERE, null, ex1);
				}
			}
		}
		else
			value = "";
		
		return value;
	}
	
	
	@Override
	public void windowOpened(WindowEvent e)
	{
//		throw new UnsupportedOperationException("Not supported yet.");
	}
	
	
	@Override
	public void windowClosing(WindowEvent e)
	{
//		throw new UnsupportedOperationException("Not supported yet.");
	}
	
	
	@Override
	public void windowClosed(WindowEvent e)
	{
//		throw new UnsupportedOperationException("Not supported yet.");
	}
	
	
	@Override
	public void windowIconified(WindowEvent e)
	{
//		throw new UnsupportedOperationException("Not supported yet.");
	}
	
	
	@Override
	public void windowDeiconified(WindowEvent e)
	{
//		throw new UnsupportedOperationException("Not supported yet.");
	}
	
	
	@Override
	public void windowActivated(WindowEvent e)
	{
		if (! programIsInitialized)
		{
			programIsInitialized = true;
			
			Thread tcpClientThread = new Thread(tcpClientComm);
			tcpClientThread.start();
			
			httpServerComm = new HTTPServerComm(this, mySignal);
			Thread httpServerThread = new Thread(httpServerComm);
			httpServerThread.start();

			ScheduledThreadPoolExecutor executor = new ScheduledThreadPoolExecutor(3);
			executor.scheduleWithFixedDelay(this, 0L, 500L, TimeUnit.MILLISECONDS);
//			executor.scheduleWithFixedDelay(this, 0L, 100L, TimeUnit.MILLISECONDS);
//			executor.scheduleAtFixedRate(this, 0L, 10L, TimeUnit.MILLISECONDS);
		}
	}
	
	
	@Override
	public void windowDeactivated(WindowEvent e)
	{
//		throw new UnsupportedOperationException("Not supported yet.");
	}

	@Override
	public void run()
	{
		mySignal.shiftText();
//		throw new UnsupportedOperationException("Not supported yet.");
	}
	
	
	// This method is thread safe
	public void addInfoMessage(final String message)
	{
		if (SwingUtilities.isEventDispatchThread())
		{
			jTextArea_Output.append(message);
		}
		else
		{
			Runnable setTextRun = new Runnable()
			{

				public void run()
				{
					jTextArea_Output.append(message);
				}
			};

			try
			{
				SwingUtilities.invokeAndWait(setTextRun);
			}
			catch (InterruptedException ex)
			{
				ex.printStackTrace();
			}
			catch (InvocationTargetException ex)
			{
				ex.printStackTrace();
			}
		}
	}
	
	
	// This method is thread safe
	public void addInfoMessageLn(String message)
	{
		addInfoMessage(message+"\n");
	}
	
	
    private void readProperties()
	{
        try
		{
            prop.load(new FileInputStream("properties.ini"));
			
			if (prop.getProperty("client_ip_address") == null)
				prop.setProperty("client_ip_address", "127.0.0.1");
            
			if (prop.getProperty("client_tcp_port") == null)
	            prop.setProperty("client_tcp_port", "2500");
			
			if (prop.getProperty("server_http_port") == null)
	            prop.setProperty("server_http_port", "80");
        }
        catch(Exception ex) {
            prop.setProperty("client_ip_address", "127.0.0.1");
            prop.setProperty("client_tcp_port", "2500");
            prop.setProperty("color", "0");
            System.out.println(ex.getMessage());
        }
		
        jTextField_ClientIPaddress.setText(prop.getProperty("client_ip_address"));
        jTextField_ClientTCPport.setText(prop.getProperty("client_tcp_port"));
        jTextField_ServerHTTPport.setText(prop.getProperty("server_http_port"));
		
		colorSetting = Integer.parseInt(prop.getProperty("color_setting","0"));
		
		
		String path = "languages";
		File[] propertiesFiles = new File(path).listFiles(new FilenameFilter()
		{

			@Override
			public boolean accept(File dir, String name)
			{
				return name.endsWith(".properties");
			}
		});


		String desiredLanguage = prop.getProperty("language", "Svenska");

		Pattern pattern;
		Matcher matcher;
		String language;
		String country;
		ResourceBundle tempResourceBundle;

		jComboBox_Language.removeAllItems();

		// \\A = början på input, \\d\\d? = en eller två siffror, : = kolon, \\z = slutet på input
		pattern = Pattern.compile("\\AMessagesBundle_(\\w\\w)_(\\w\\w)\\.properties\\z");

		if (propertiesFiles != null)
		{
			for (int i = 0; i < propertiesFiles.length; i++)
			{
				matcher = pattern.matcher(propertiesFiles[i].getName());

				if (matcher.matches())
				{
					language = matcher.group(1);
					country = matcher.group(2);

					ClassLoader loader = null;
					try
					{
						String absolutePath = propertiesFiles[i].getAbsolutePath();
						String filePath = absolutePath.substring(0, absolutePath.lastIndexOf(File.separator));
						URL[] urls =
						{
							new File(filePath).toURI().toURL()
						};
						loader = new URLClassLoader(urls);
						tempResourceBundle = ResourceBundle.getBundle("MessagesBundle", new Locale(language, country), loader);
						localizationResourceBundleList.add(tempResourceBundle);
						
						jComboBox_Language.addItem(tempResourceBundle.getString("language_name"));
						
						if (desiredLanguage.equals(tempResourceBundle.getString("language_name")))
						{
							currentLocalizationResourceBundle = tempResourceBundle;
							currentLanguage = tempResourceBundle.getString("language_name");
							jComboBox_Language.setSelectedIndex(jComboBox_Language.getItemCount()-1);
						}
					}
					catch (MalformedURLException ex)
					{
//						addInfoMessageLn(StringUtils.getStackTrace(ex));
					}
				}
				else
				{
					try
					{
//						jTextArea_Output.append("Filen " + propertiesFiles[i].toString() + " har ett felaktigt filnamn\n");
						mySignal.debug.write("Filen " + propertiesFiles[i].toString() + " har ett felaktigt filnamn\n");
					}
					catch (IOException ex)
					{
//						Logger.getLogger(DBFastClock.class.getName()).log(Level.SEVERE, null, ex);
					}
				}
			}

			if ( (currentLanguage == null) && (localizationResourceBundleList.size() > 0) )
			{
				currentLocalizationResourceBundle = localizationResourceBundleList.get(0);
				currentLanguage = currentLocalizationResourceBundle.getString("language_name");
			}
		}
		else
		{
			currentLanguage = null;
//			jTextArea_Output.append("No language files found\n");
		}
		
    }

	
    public void writeProperties()
	{
        try
		{
//            if (jComboBoxSerialPort.getSelectedIndex() != -1)
//                prop.setProperty("com_port",jComboBoxSerialPort.getSelectedItem().toString());
//            else
//                prop.setProperty("com_port","");

            prop.store(new FileOutputStream("properties.ini"), "Settings");
        }
        catch(IOException ioe) {
            System.out.println(ioe.getMessage());
        }
    }


}
